/*
 *    wmupmon - A dockapp to monitor system uptime
 *    Copyright (C) 2003  Justin Spadea <jzs@mail.rit.edu>
 *
 *    Based on work by Seiichi SATO <ssato@sh.rim.or.jp>
 *    Copyright (C) 2001,2002  Seiichi SATO <ssato@sh.rim.or.jp>
 *    AND by Mark Staggs <me@markstaggs.net>
 *    Copyright (C) 2002  Mark Staggs <me@markstaggs.net>

 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.

 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.

 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include "dockapp.h"
#include <dockapp.h>
#include "backlight_on.xpm"
#include "backlight_off.xpm"
#include "backlight_off_stats.xpm"
#include "backlight_on_stats.xpm"
#include "display_error.xpm"
#include "parts.xpm"
#include "font.xpm"
#include "wmwifi.h"

#define SIZE	    58
#define WINDOWED_BG "  \tc #AEAAAE"
#define MAX_HISTORY 16
#define CPUNUM_NONE -1

typedef enum { LIGHTON, LIGHTOFF } light;
typedef enum { STANDARD, EXTENDED } interface;
Pixmap pixmap;
Pixmap fonts;
Pixmap backdrop_on;
Pixmap backdrop_off;
Pixmap backdrop_off_stats;
Pixmap backdrop_on_stats;
Pixmap display_error;
Pixmap parts;
Pixmap mask;
static char *display_name = "";
static char *light_color = NULL;	/* back-light color */
static unsigned update_interval = 1;
static light backlight = LIGHTOFF;
static interface gui = STANDARD;
Bool wmwifi_learn;
int count;


/* prototypes */
static void update(struct wifi *wfi);
static void switch_light(struct wifi *wfi);
static void switch_interface(struct wifi *wfi);
static void draw_junk(struct wifi *wfi);
static void draw_ext(struct wifi *wfi);
static void draw_num(int num, int dx, int dy);
static void draw_text(char *text, int dx, int dy);
static void parse_arguments(int argc, char **argv);
static void print_help(char *prog);

int main(int argc, char **argv)
{
    XEvent event;
    XButtonPressedEvent *bevent;
    XpmColorSymbol colors[2] = { {"Back0", NULL, 0}, {"Back1", NULL, 0} };
    int ncolor = 0;
    int stuff = 1, i;
    struct wifi wfi;

    wfi.ifnum = 0;

    /* Parse CommandLine */
    parse_arguments(argc, argv);

    /* Initialize Application */
    dockapp_open_window(display_name, PACKAGE, SIZE, SIZE, argc, argv);
    dockapp_set_eventmask(ButtonPressMask);

    if (light_color) {
	colors[0].pixel = dockapp_getcolor(light_color);
	colors[1].pixel =
	    dockapp_blendedcolor(light_color, -24, -24, -24, 1.0);
	ncolor = 2;
    }

    /* change raw xpm data to pixmap */
    if (dockapp_iswindowed)
	backlight_on_xpm[1] = backlight_off_xpm[1] = WINDOWED_BG;

    if (!dockapp_xpm2pixmap
	(backlight_on_xpm, &backdrop_on, &mask, colors, ncolor)) {
	fprintf(stderr, "Error initializing backlit background image.\n");
	exit(1);
    }
    if (!dockapp_xpm2pixmap
	(backlight_off_xpm, &backdrop_off, NULL, NULL, 0)) {
	fprintf(stderr, "Error initializing background image.\n");
	exit(1);
    }
    if (!dockapp_xpm2pixmap
	(backlight_off_stats_xpm, &backdrop_off_stats, NULL, NULL, 0)) {
	fprintf(stderr, "Error initializing background image.\n");
	exit(1);
    }
    if (!dockapp_xpm2pixmap
	(backlight_on_stats_xpm, &backdrop_on_stats, NULL, NULL, 0)) {
	fprintf(stderr, "Error initializing background image.\n");
	exit(1);
    }
    if (!dockapp_xpm2pixmap
	(display_error_xpm, &display_error, NULL, NULL, 0)) {
	fprintf(stderr, "Error initializing error image.\n");
	exit(1);
    }
    if (!dockapp_xpm2pixmap(parts_xpm, &parts, NULL, colors, ncolor)) {
	fprintf(stderr, "Error initializing parts image.\n");
	exit(1);
    }
    if (!dockapp_xpm2pixmap(font_xpm, &fonts, NULL, colors, ncolor)) {
	fprintf(stderr, "Error initializing fonts image.\n");
	exit(1);
    }
    /* shape window */
    if (!dockapp_iswindowed)
	dockapp_setshape(mask, 0, 0);
    if (mask)
	XFreePixmap(display, mask);

    /* pixmap : draw area */
    pixmap = dockapp_XCreatePixmap(SIZE, SIZE);

    /* Initialize pixmap */
    if (backlight == LIGHTON)
	dockapp_copyarea(backdrop_on, pixmap, 0, 0, SIZE, SIZE, 0, 0);
    else
	dockapp_copyarea(backdrop_off, pixmap, 0, 0, SIZE, SIZE, 0, 0);

    if (gui == STANDARD)
	dockapp_copyarea(backdrop_off, pixmap, 0, 0, SIZE, SIZE, 0, 0);
    else
	dockapp_copyarea(backdrop_off_stats, pixmap, 0, 0, SIZE, SIZE, 0,
			 0);

    dockapp_set_background(pixmap);
    dockapp_show();

    /* Main loop */
    //while (1) {
    for (i = 0;; i++) {
	/* CHANGED */
	if (dockapp_nextevent_or_timeout(&event, update_interval * 500)) {
	    /* Next Event */
	    switch (event.type) {
	    case ButtonPress:
		bevent = (XButtonPressedEvent *) & event;
		switch (bevent->button & 0xff) {
		case Button1:
		    switch_light(&wfi);
		    break;
		case Button2:
		    next_if(&wfi);
		    break;
		case Button3:
		    //last_if(&wfi);
		    switch_interface(&wfi);
		    break;
		}

		break;
	    default:		/* make gcc happy */
		break;
	    }
	} else {
	    /* Time Out */
	    update(&wfi);
	}
    }

    return 0;
}

/* called by timer */
static void update(struct wifi *wfi)
{
    int sw;
    extern int count;
    static light pre_backlight;

    /* get current link level from /proc/net/wireless */
    get_wifi_info(wfi);

    if (wfi->max_qual == -1) {
	dockapp_copyarea(display_error, pixmap, 0, 0, 58, 58, 0, 0);
    } else {
	/* all clear */

	if (gui == STANDARD) {
	    if (backlight == LIGHTON) {
		dockapp_copyarea(backdrop_on, pixmap, 0, 0, 58, 58, 0, 0);
	    } else {
		dockapp_copyarea(backdrop_off, pixmap, 0, 0, 58, 58, 0, 0);
	    }
	    /* draw junk */
	    draw_junk(wfi);
	} else {
	    if (backlight == LIGHTON) {
		dockapp_copyarea(backdrop_on_stats, pixmap, 0, 0, 58, 58,
				 0, 0);
	    } else {
		dockapp_copyarea(backdrop_off_stats, pixmap, 0, 0, 58, 58,
				 0, 0);
	    }
	    draw_ext(wfi);
	}
	count++;
	if (count < 5) {
	    sw = 1;
	}
	if (count >= 5) {
	    sw = 2;
	}
	if (count == 10) {
	    count = -1;
	}
	switch (sw) {
	case 1:
	    //draw_ifname(wfi);
	    draw_text(wfi->ifname, ((56 / 2) - (strlen(wfi->ifname) * 2)),
		      4);
	    break;
	case 2:
	    //draw_essid(wfi);
	    draw_text(wfi->essid, ((56 / 2) - (strlen(wfi->essid) * 2)),
		      4);
	    break;
	}
	/* show */
    }
    dockapp_copy2window(pixmap);
}
static void switch_interface(struct wifi *wfi)
{
    switch (gui) {
    case STANDARD:
	gui = EXTENDED;
	if (backlight == LIGHTON) {
	    dockapp_copyarea(backdrop_on, pixmap, 0, 0, 58, 58, 0, 0);
	} else {
	    dockapp_copyarea(backdrop_off, pixmap, 0, 0, 58, 58, 0, 0);
	}
	break;
    case EXTENDED:
	gui = STANDARD;
	if (backlight == LIGHTON) {
	    dockapp_copyarea(backdrop_on_stats, pixmap, 0, 0, 58, 58, 0,
			     0);
	} else {
	    dockapp_copyarea(backdrop_off_stats, pixmap, 0, 0, 58, 58, 0,
			     0);
	}
	break;

    }
    dockapp_copy2window(pixmap);
}

/* called when mouse button pressed */
static void switch_light(struct wifi *wfi)
{
    int link;

    switch (backlight) {
    case LIGHTOFF:
	backlight = LIGHTON;
	if (gui == STANDARD) {
	    dockapp_copyarea(backdrop_on, pixmap, 0, 0, 58, 58, 0, 0);
	} else {
	    dockapp_copyarea(backdrop_on_stats, pixmap, 0, 0, 58, 58, 0,
			     0);
	}
	break;
    case LIGHTON:
	backlight = LIGHTOFF;
	if (gui == STANDARD) {
	    dockapp_copyarea(backdrop_off, pixmap, 0, 0, 58, 58, 0, 0);
	} else {
	    dockapp_copyarea(backdrop_off_stats, pixmap, 0, 0, 58, 58, 0,
			     0);
	}
	break;
    }

    /* get current current link level from /proc/net/wireless */
    get_wifi_info(wfi);

    /* redraw junk */
    //draw_ifname(wfi);
    if (gui == STANDARD) {
	draw_junk(wfi);
    } else {
	draw_ext(wfi);
    }

    /* show */
    dockapp_copy2window(pixmap);
}
static void draw_ext(struct wifi *wfi)
{
    char buffer[25];
    double rate = wfi->bitrate.value;

    draw_text("link", (56 - (strlen("link") * 5)), 50);
    draw_text("level", (56 - (strlen("level") * 5)), 40);
    draw_text("noise", (56 - (strlen("noise") * 5)), 30);
    draw_text("rate", (56 - (strlen("rate") * 5)), 20);
    draw_num(wfi->link, 12, 47);
    draw_num(wfi->level - 0x100, 12, 37);
    draw_num(wfi->noise - 0x100, 12, 27);
    if (rate >= GIGA) {
	draw_num(rate / GIGA, 0, 17);
    } else {
	if (rate >= MEGA) {
	    draw_num(rate / MEGA, 0, 17);
	} else {
	    draw_num(rate / KILO, 0, 17);
	}
    }

}
static void draw_text(char *text, int dx, int dy)
{
    int ax, ay = 0, bx, len, i;

    len = strlen(text);
    bx = 4;

    if (backlight == LIGHTON) {
	ay = 5;
    }

    for (i = 0; i < len; i++) {
	if (isalpha(text[i])) {
	    ax = (tolower(text[i]) - 97) * 4;
	} else {
	    ax = ((text[i] - 47) * 4) + 100;
	}
	dockapp_copyarea(fonts, pixmap, ax, ay, 4, 5, dx, dy);
	dx += 5;
    }

}
static void draw_num(int num, int dx, int dy)
{
    int v100, v10, v1, v2;
    int percent;
    int onum = 0;
    int d1, d2, d3;
    int s;
    int y = 40, x1 = 0, x2 = 0, x3 = 0, x4 = 0, y1 = 5;

    if (num < 0) {
	/* convert num to positive, save real number in onum */
	onum = num;
	num = num * -1;
    }

    if (num > 99) {
	v2 = num / 100;
	v10 = (num - (v2 * 100)) / 10;
	v1 = (num - (v2 * 100)) - (v10 * 10);
    } else {
	v10 = num / 10;
	v1 = num - (v10 * 10);
    }

    x1 = v10 * 5;
    x2 = v1 * 5;
    x3 = v2 * 5;

    if (backlight == LIGHTON) {
	x1 += 50;
	x2 += 50;
	x3 += 50;
	x4 += 50;
	y1 = 36;
    }

    if (onum < 0) {
	/* If we get a negative number, draw the dash */
	if (num > 99) {
	    s = dx - 6;
	} else if (num > 9) {
	    s = dx;
	} else {
	    s = dx + 6;
	}
	dockapp_copyarea(parts, pixmap, x4, 71, 5, 9, s, dy);
    }
    /* draw digits */
    if (num > 99) {
	dockapp_copyarea(parts, pixmap, x3, 62, 5, 9, dx, dy);
    }
    if (num > 9) {
	dockapp_copyarea(parts, pixmap, x1, 62, 5, 9, dx + 6, dy);
    }
    dockapp_copyarea(parts, pixmap, x2, 62, 5, 9, dx + 12, dy);
}

static void draw_junk(struct wifi *wfi)
{
    int v100, v10, v1, num = wfi->link;
    int percent;
    int y = 40, x1 = 0, x2 = 0, x3 = 5, y1 = 5;

    if (backlight == LIGHTON) {
	y1 = 36;
    } else {
	y1 = 5;
    }
    if (wfi->max_qual > -1) {
	if (num < 0)
	    num = 0;

	/* Calculate Link percentage */
	percent = num / (wfi->max_qual / 100);
	num = percent;

	/* draw digits */
	draw_num(num, 6, 47);

	if (num > 0) {
	    /* Level bar 1 ( > 1 - 20%) */
	    dockapp_copyarea(parts, pixmap, 70, 27, 2, 2, 37, 44);
	}
	if (num > 20) {
	    /* Level bar 2 (18.4 - 40%) */
	    dockapp_copyarea(parts, pixmap, 70, 27, 2, 2, 41, 44);
	    dockapp_copyarea(parts, pixmap, 74, 22, 2, 7, 41, 37);
	    /* Antenna */
	    dockapp_copyarea(parts, pixmap, 2, y1, 15, 24, 10, 21);
	}
	if (num > 40) {
	    /* Level bar 3 (36.8 - 60%) */
	    dockapp_copyarea(parts, pixmap, 70, 27, 2, 2, 45, 44);
	    dockapp_copyarea(parts, pixmap, 74, 22, 2, 7, 45, 37);
	    dockapp_copyarea(parts, pixmap, 74, 22, 2, 7, 45, 30);
	    /* Antenna */
	    dockapp_copyarea(parts, pixmap, 18, y1, 15, 24, 10, 21);
	}
	if (num > 60) {
	    dockapp_copyarea(parts, pixmap, 70, 27, 2, 2, 49, 44);
	    dockapp_copyarea(parts, pixmap, 74, 22, 2, 7, 49, 37);
	    dockapp_copyarea(parts, pixmap, 74, 22, 2, 7, 49, 30);
	    dockapp_copyarea(parts, pixmap, 74, 22, 2, 7, 49, 23);
	    /* Antenna */
	    dockapp_copyarea(parts, pixmap, 34, y1, 15, 24, 10, 21);
	}
	if (num > 80) {
	    /* Level bar 5 (73.6 - 100%) */
	    dockapp_copyarea(parts, pixmap, 70, 27, 2, 2, 53, 44);
	    dockapp_copyarea(parts, pixmap, 74, 22, 2, 7, 53, 37);
	    dockapp_copyarea(parts, pixmap, 74, 22, 2, 7, 53, 30);
	    dockapp_copyarea(parts, pixmap, 74, 22, 2, 7, 53, 23);
	    dockapp_copyarea(parts, pixmap, 74, 22, 2, 7, 53, 16);
	    /* Antenna */
	    dockapp_copyarea(parts, pixmap, 50, y1, 15, 24, 10, 21);
	}
    }

}

static void parse_arguments(int argc, char **argv)
{
    int i;
    int integer;
    for (i = 1; i < argc; i++) {
	if (!strcmp(argv[i], "--help") || !strcmp(argv[i], "-h"))
	    print_help(argv[0]), exit(0);
	else if (!strcmp(argv[i], "--version") || !strcmp(argv[i], "-v"))
	    printf("%s version %s\n", PACKAGE, VERSION), exit(0);
	else if (!strcmp(argv[i], "--display") || !strcmp(argv[i], "-d")) {
	    display_name = argv[i + 1];
	    i++;
	} else if (!strcmp(argv[i], "--backlight")
		   || !strcmp(argv[i], "-bl"))
	    backlight = LIGHTON;
	else if (!strcmp(argv[i], "--light-color")
		 || !strcmp(argv[i], "-lc")) {
	    light_color = argv[i + 1];
	    i++;
	} else if (!strcmp(argv[i], "--interval")
		   || !strcmp(argv[i], "-i")) {
	    if (argc == i + 1)
		fprintf(stderr,
			"%s: error parsing argument for option %s\n",
			argv[0], argv[i]), exit(1);
	    if (sscanf(argv[i + 1], "%i", &integer) != 1)
		fprintf(stderr,
			"%s: error parsing argument for option %s\n",
			argv[0], argv[i]), exit(1);
	    if (integer < 1)
		fprintf(stderr, "%s: argument %s must be >=1\n",
			argv[0], argv[i]), exit(1);
	    update_interval = integer;
	    i++;
	} else if (!strcmp(argv[i], "--learn")
		   || !strcmp(argv[i], "-l")) {
	    wmwifi_learn = True;
	} else if (!strcmp(argv[i], "--windowed")
		   || !strcmp(argv[i], "-w")) {
	    dockapp_iswindowed = True;
	} else if (!strcmp(argv[i], "--broken-wm")
		   || !strcmp(argv[i], "-bw"))
	    dockapp_isbrokenwm = True;
	else {
	    fprintf(stderr, "%s: unrecognized option '%s'\n", argv[0],
		    argv[i]);
	    print_help(argv[0]), exit(1);
	}
    }
}

static void print_help(char *prog)
{
    printf("Usage : %s [OPTIONS]\n", prog);
    printf("wmupmon - uptime monitor dockapp\n");
    printf("  -d,  --display <string>        display to use\n");
    printf("  -bl, --backlight               turn on back-light\n");
    printf
	("  -lc, --light-color <string>    back-light color(rgb:6E/C6/3B is default)\n");
    printf
	("  -i,  --interval <number>       number of secs between updates (1 is default)\n");
    printf
	("  -h,  --help                    show this help text and exit\n");
    printf
	("  -v,  --version                 show program version and exit\n");
    printf
	("  -w,  --windowed                run the application in windowed mode\n");
    printf
	("  -bw, --broken-wm               activate broken window manager fix\n");
}
